package ru.dbotthepony.mc.otm.screen;

import com.mojang.blaze3d.systems.RenderSystem;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.client.gui.screens.inventory.AbstractContainerScreen;
import net.minecraft.client.renderer.GameRenderer;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.FormattedText;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.inventory.Slot;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.menu.FormattingHelper;
import ru.dbotthepony.mc.otm.menu.MatteryMenu;
import ru.dbotthepony.mc.otm.menu.slot.BatterySlot;
import ru.dbotthepony.mc.otm.menu.slot.MachineInputSlot;
import ru.dbotthepony.mc.otm.menu.slot.MachineOutputSlot;
import ru.dbotthepony.mc.otm.menu.slot.MatterContainerInputSlot;
import ru.dbotthepony.mc.otm.menu.widget.AbstractWidget;

import java.math.BigDecimal;
import java.util.List;
import java.util.function.Supplier;

public class MatteryScreen<T extends MatteryMenu> extends AbstractContainerScreen<T> {
	protected static final ResourceLocation CONTAINER_BASE = new ResourceLocation(OverdriveThatMatters.MOD_ID, "textures/gui/generic_machine.png");
	public static final ResourceLocation WIDGETS = new ResourceLocation(OverdriveThatMatters.MOD_ID, "textures/gui/widgets.png");

	protected ResourceLocation CONTAINER_BACKGROUND() {
		return CONTAINER_BASE;
	}

	protected int render_x;
	protected int render_y;

	public MatteryScreen(T p_97741_, Inventory p_97742_, Component p_97743_) {
		super(p_97741_, p_97742_, p_97743_);

		imageWidth = 176;
		imageHeight = 178;
		titleLabelY = 4;

		inventoryLabelY = imageHeight - 92;
	}

	@Override
	protected void renderTooltip(PoseStack pose, int mouseX, int mouseY) {
		super.renderTooltip(pose, mouseX, mouseY);

		if (this.hoveredSlot == null && menu.getCarried().isEmpty()) {
			for (AbstractWidget widget : menu.widget_list) {
				if (widget.renderTooltip(this, pose, render_x, render_y, mouseX, mouseY)) {
					return;
				}
			}
		}
	}

	@Override
	public void render(PoseStack p_98418_, int mouseX, int mouseY, float p_98421_) {
		render_x = (width - imageWidth) / 2;
		render_y = (height - imageHeight) / 2;

		for (AbstractWidget<?> widget : menu.widget_list) {
			widget.updateIsHovered(render_x, render_y, mouseX, mouseY);
		}

		this.renderBackground(p_98418_);
		super.render(p_98418_, mouseX, mouseY, p_98421_);
		this.renderTooltip(p_98418_, mouseX, mouseY);
	}

	protected void renderRegularSlot(PoseStack pose, int x, int y) {
		this.blit(pose, render_x + x - 1, render_y + y - 1, 0, 96, 18, 18);
	}

	protected void renderOutputSlot(PoseStack pose, int x, int y) {
		this.blit(pose, render_x + x - 5, render_y + y - 5, 18, 96, 26, 26);
	}

	protected void renderBatterySlot(PoseStack pose, int x, int y) {
		renderRegularSlot(pose, x, y);
		this.blit(pose, render_x + x - 1, render_y + y - 1, 0, 114, 18, 18);
	}

	@Override
	protected void renderBg(PoseStack pose, float p_97788_, int mouseX, int mouseY) {
		RenderSystem.setShader(GameRenderer::getPositionTexShader);
		RenderSystem.setShaderColor(1.0F, 1.0F, 1.0F, 1.0F);
		RenderSystem.setShaderTexture(0, CONTAINER_BACKGROUND());
		// PoseStack, x, y, image_start_x, image_start_y, rect_size_x, rect_size_y
		this.blit(pose, render_x, render_y, 0, 0, this.imageWidth, this.imageHeight);

		RenderSystem.setShader(GameRenderer::getPositionTexShader);
		RenderSystem.setShaderColor(1.0F, 1.0F, 1.0F, 1.0F);
		RenderSystem.setShaderTexture(0, WIDGETS);

		for (Slot slot : menu.slots) {
			if (slot instanceof BatterySlot slot1 && slot1.auto_bg) {
				renderBatterySlot(pose, slot1.x, slot1.y);
			} else if (slot instanceof MachineOutputSlot slot1 && slot1.auto_bg) {
				if (slot1.is_main_output) {
					renderOutputSlot(pose, slot1.x, slot1.y);
				} else {
					renderRegularSlot(pose, slot1.x, slot1.y);
				}
			} else if (slot instanceof MachineInputSlot slot1 && slot1.auto_bg) {
				renderRegularSlot(pose, slot1.x, slot1.y);
			} else if (slot instanceof MatterContainerInputSlot slot1 && slot1.auto_bg) {
				renderRegularSlot(pose, slot1.x, slot1.y);
			}
		}

		for (AbstractWidget widget : menu.widget_list) {
			widget.renderBackground(this, pose, render_x, render_y, mouseX, mouseY);
		}
	}
}
